module PandaPal
  module OrganizationConcerns
    module MultiDatabaseSharding
      extend ActiveSupport::Concern

      included do
        if column_names.include?('shard')
          validates :shard, format: { with: /\A[a-z0-9_]+\z/i }, allow_blank: true

          before_validation on: [:update] do
            errors.add(:shard, 'should not be changed after creation') if shard_changed?
          end
        end
      end

      def tenant_name
        return "#{shard}:#{name}" if shard.present?
        super
      end
    end
  end
end
