# frozen_string_literal: true

module PandaPal::Concerns
  module AbilityHelper
    extend ActiveSupport::Concern

    class_methods do
      def cache_on_session(method_sym)
        mthd = instance_method(mthd_sym)
        pmthd_sym = :"_scache_#{mthd_sym}"

        alias_method pmthd_sym, mthd_sym
        private pmthd_sym

        class_eval <<~RUBY
          def #{method_sym}()
            cache_on_session(:#{pmthd_sym}) do
              #{pmthd_sym}()
            end
          end
        RUBY

        method_sym
      end
    end

    # @return PandaPal::Session
    def panda_pal_session
      unless defined?(@panda_pal_session) || @controller.present? || method(:panda_pal_session).owner == PandaPal::Concerns::AbilityHelper
        raise "Ability class needs to set @panda_pal_session or @controller to use this feature"
      end

      @panda_pal_session = @controller.current_panda_session unless defined?(@panda_pal_session)

      if @panda_pal_session.is_a?(Hash)
        # This is a breaking-change to CanvasSync, but not to PandaPal
        raise "Ability#panda_pal_session/@panda_pal_session should now return a PandaPal::Session object, not a Hash"
      end

      @panda_pal_session
    end

    def rails_session
      unless defined?(@rails_session) || @controller.present? || method(:rails_session).owner == PandaPal::Concerns::AbilityHelper
        raise "Ability class needs to set @rails_session or @controller to use this feature"
      end

      @rails_session ||= @controller.session unless defined?(@rails_session)
    end

    # Delegate legacy AbilityHelper methods to panda_pal_session. TODO Should such direct usage be considered deprecated?
    delegate :launch_params, :get_lti_cust_param, :lti_launch_placement, :canvas_site_admin?, :lti_roles, :canvas_role_labels, :canvas_account_role_labels, to: :panda_pal_session

    def is_lti_launch?
      panda_pal_session.present?
    end

    def cache_on_session(*args, **kwargs, &blk)
      panda_pal_session.cache(*args, **kwargs, &blk)
    end
  end
end
